/*
    simulator.c
    Copyright (C) 2014
    Authors: Arun I, Hari Prasad Gokul R 

    This file is part of Quiz Buzzer v2 firmware.

    Quiz Buzzer v2 firmware is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Quiz Buzzer v2 firmware is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Quiz Buzzer v2 firmware.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <simavr/sim_avr.h>
#include <simavr/sim_elf.h>
#include <simavr/sim_vcd_file.h>
#include <simavr/avr_ioport.h>
#include <pthread.h>

#include "peripherals.h"
#include "simulator.h"

avr_t* avr = NULL; 
avr_vcd_t vcd_file;

static void* avr_thread(void* ignore);

void simulator_init()
{
	elf_firmware_t f;
	const char* fname = "../src/buzzerv2.elf";
	elf_read_firmware(fname, &f);

	avr = avr_make_mcu_by_name("atmega8");
	avr_init(avr);
	avr_load_firmware(avr, &f);

	peripherals_init(avr);

	avr_vcd_init(avr, "gtkwave_output.vcd", &vcd_file, 100000);
	avr_vcd_add_signal(&vcd_file, avr_io_getirq(avr, AVR_IOCTL_IOPORT_GETIRQ('D'), IOPORT_IRQ_PIN_ALL), 8, "switches");
	avr_vcd_add_signal(&vcd_file, avr_io_getirq(avr, AVR_IOCTL_IOPORT_GETIRQ('C'), 5), 1, "reset");
	avr_vcd_add_signal(&vcd_file, avr_io_getirq(avr, AVR_IOCTL_IOPORT_GETIRQ('B'), IOPORT_IRQ_PIN_ALL), 8, "display");
}

void simulator_start()
{
	pthread_t avrrun;
	avr_vcd_start(&vcd_file);
	pthread_create(&avrrun, NULL, avr_thread, NULL);
}

void simulator_stop()
{
	avr_vcd_stop(&vcd_file);
}

static void* avr_thread(void* ignore)
{
	int state = cpu_Running;
	while ((state != cpu_Done) && (state != cpu_Crashed))
		state = avr_run(avr);
	return NULL;
}
